/* camp.c: calls an assembly module to install a tsr that
just camps out using up memory. Uninstaller is also installed that
responds to INT 01H. Amount of memory to use can be given on command line. 
*/


#include <stdlib.h>
#include <stdio.h>
#include <string.h>

#define DEFAULT_PAR 1024     
#define MAX_PAR 32768
#define INFO  "\ncamp: allocate memory and stay resident\n\n\
usage:  camp [/help /u /a /q ] <paragraphs to keep> \n\
   where /h[elp] or /? prints this message,\n\
         /u uinstalls the tsr and frees memory, \n\
         /q quiet operation \n\
	 /a allocate at highest possible address\n\
   otherwise memory allocated just above non-resident\n\
   part of the program.\
"   
		
#define MAX_ARG_LEN 16             /* Max length of combined options */
#define USAGE "Usage: camp [ /help /u /q > ] <paragraphs to keep>\n"

/* option processing code */
int find_opt(char *option_string);
int setcmp(const char *, const char *, char);

/* defined in camp.asm */
extern void setup(unsigned int par);
extern unsigned char quiet;
extern void uninstall(void);
extern void set_umb_link(void);
extern void restore_umb_link(void);
extern int strategy;
extern unsigned char LinkFlag;

int
main(int argc, char **argv)
{
	unsigned par = DEFAULT_PAR;
	int c=-1;                /* numeric option code */
	char *info = INFO;
	char argstr[MAX_ARG_LEN] = "/";

	strategy = 0x00; /* default is to camp at the lowest
				possible address */

	if(--argc > 0){  /* Anything on command line ? */
		if((*++argv)[0]=='/'){      /* already the end of options */
			if((*argv)[1]=='/'){
				++argv;
				c=-1;  /* no options */
			}
			else strcat(argstr,++*argv); /* start building argstr*/
			argc--;
			while((argc>0)&&((*++argv)[0]=='/')&&((*argv)[1]!='/')){
                   /* combine all options into a single string */
				argc--;
				strcat(argstr,++*argv);
			}
                        c=find_opt(argstr); /* look up code, set flags */
		}
		switch(c){

			case -1: break;  /* default, e.g no options */

			case 0: printf("%s\n",info);
				return 0;

			case 1: 
				set_umb_link();  
				uninstall();
				restore_umb_link();
				if(!quiet)
					printf("Uninstalled. Memory freed.\n");
				return 0;

			case 3:  /* load high */
				strategy = 0x82;
				break;

                        case 2: 
                                break;
			case -2: ;  /* barf (don't use in your code) */
			default:
				fprintf(stderr,"%s: illegal option\n",argstr);
				return 1;
			}
	}
	if(argc > 0)par = atoi(*argv);
        if(par <= 0){
		  fprintf(stderr,USAGE);
		  return 1;
	}
	if(par > MAX_PAR) {  /* Too big */
	fprintf(stderr,"camp: request paragraphs cannot exceed %u.\n",MAX_PAR);
		return 1;
	}


        if(!quiet){

	         /* Print out information on what has happened */

		printf("Camping on %u paragraphs (%u bytes) of memory\n",par,
			par*16);
		printf("Use camp /u to uninstall and free memory\n");
       }

	/* Requested paragraphs must be reduced by 1 to allow for the
           memory control block (aka arena header) that will take up
           16 bytes = 1 par ahead of the psp of the installed tsr */

        set_umb_link(); 
        setup(par-1);  /* Do it! */
        restore_umb_link(); 

	return 0; 
}

/*  find_opt: return a unique small integer for each possible option string */
/*  There should be a case in the switch statement in main to handle each. */
/*  -2 reserved for arg not found -1 reserved for no options */

int find_opt(char *word)
{
	if(strcmp(word,"/help")==0)return 0;
	if(strcmp(word,"/?")==0)return 0;
	if(strcmp(word,"/h")==0)return 0;
	if(strcmp(word,"/H")==0)return 0;
	if(strcmp(word,"/u")==0)return 1;
	if(strcmp(word,"/a")==0)return 3;
	if(strcmp(word,"/U")==0)return 1;
        if(strcmp(word,"/q")==0){
                quiet = 1;
                return 2;
        }
        if(setcmp(word,"/qu",'\1')){
                quiet = 1;
                return 1;
        }
        if(setcmp(word,"/qa",'\1')){
                quiet = 1;
                return 3;
        }
/* args not found */
	return -2;
}


/* setcmp: string function that returns 1 if s and t are equal as sets,
           i.e., contain the same characters, not necessarily in the
           same order. Returns 0 if not, or if the answer cannot be
           determined. The invld character is any character not present
           in either s or t.  */

int setcmp(const char *s, const char *t, char invld)
{
        int i;
        char c;
        char *ptr;
        char buffer[MAX_ARG_LEN];

        if(s == NULL && t == NULL) return 1;       /* trivial case */
        if( t == NULL ) return 0; /* since here s must not be null */
        if(strchr(t,invld)!=NULL) {
                fprintf(stderr,"setcmp: error: character %c occurs in string %s\n",
                        invld,t);
               return 0;
        }
        if(strlen(t) != strlen(s)) return 0;

        /* From now on, both s and t are non-empty and of the same length */
        strcpy(buffer,t);
        for(i=0;i<strlen(s);i++){  /* loop over chars in s */
                c = *(s+i);  /* get next char of s */
                if((ptr=strchr(buffer,c))==NULL) return 0; /* No c in t */
                *ptr = invld;    /* else replace first occurence of c with
                                     invld so it won't be found again */
        }
        return 1; /* OK if we got to here */
}


